module mseawater
use mkinds


real(r4), parameter :: ahalf=1.0/2.0
real(r4), parameter :: a3rd =1.0/3.0
real(r4), parameter :: athird =1.0/3.0
real(r4), parameter :: a4th =1.0/4.0
real(r4), parameter :: afourth=1.0/4.0

contains

real(r4) function atg (s, t, p)

! DESCRIPTION:  adiabatic temperature gradient (deg C per decibar)

! REFERENCES:   Bryden, H. (1973), Deep-Sea Res., 20, 401-408

! UNITS:        pressure         p0        decibars
!               temperature      to        deg celsius (ipts-68)
!               salinity         s         (ipss-78)
!               reference prs    pr        decibars
!               potential tmp    theta     deg celsius
!
! CHECK VALUE:  atg = 3.255976e-4 deg C/dbar for
!               s = 40 (ipss-78), t=40 deg C, p = 10000 decibars

! PARAMETERS:
!      Name        Type        Usage           Description
!   ----------   ---------    -------    -------------------------
!       p          real        input     pressure in decibars
!       t          real        input     temperature (celsius)
!       s          real        input     salinity (pss-78)

implicit  none

real(r4) ::  ds
real(r4) ::  p, s, t


ds = s - 35.
atg = (((-2.1687e-16 * t + 1.8676e-14) * t - 4.6206e-13) * &
          p + ((2.7759e-12 * t - 1.1351e-10) * ds + ((-5.4481e-14 * &
          t + 8.733e-12) * t - 6.7795e-10) * t + 1.8741e-8)) * &
          p + (-4.2393e-8 * t + 1.8932e-6) * ds + ((6.6228e-10 * &
          t - 6.836e-8) * t + 8.5258e-6) * t + 3.5803e-5

end Function atg


real(r4) function theta (s, t0, p0, pr)

!DESCRIPTION:  computes local potential temperature at "pr" using
!              Bryden 1973 polynomial for adiabatic lapse rate
!              and Runge-Kutta 4th order integration algorithm

!REFERENCES:   Bryden, H. (1973), Deep-Sea Res., 20, 401-408
!              Fofonoff, N. (1977), Deep-Sea Res., 24, 489-491

!UNITS:        pressure         p0        decibars
!              temperature      to        deg celsius (ipts-68)
!              salinity         s         (ipss-78)
!              reference prs    pr        decibars
!              potential tmp    theta     deg celsius

!CHECK VALUE:  theta = 36.89073 deg C for
!              s = 40 (ipss-78), t0 = 40 deg C,
!              p0 = 10000 decibars, pr = 0 decibars

      
!PARAMETERS:
!      Name        Type        Usage           Description
!   ----------   ---------    -------    -------------------------
!      p0          real        input     pressure in decibars
!      t0          real        input     temperature (celsius)
!       s          real        input     salinity (pss-78)
!      pr          real        input     reference pressure (decibars)


implicit  none

real(r4) ::      h
real(r4) ::      p, p0, pr
real(r4) ::      q, s
real(r4) ::      t, t0
real(r4) ::      xk

 
! ..set up intermediate temperature and pressure variables

      p = p0
      t = t0

!     ..solve for potential temperature

      h = pr - p
      xk = h * atg (s, t, p)
      t = t + 0.5 * xk
      q = xk
      p = p + 0.5 * h     
      xk = h * atg (s, t, p)
      t = t + 0.29289322 * (xk - q)
      q = 0.58578644 * xk + 0.121320344 * q
      xk = h * atg (s, t, p)
      t = t + 1.707106781 * (xk - q)
      q = 3.414213562 * xk - 4.121320344 * q
      p = p + 0.5 * h
      xk = h * atg (s, t, p)
      theta = t + (xk - 2. * q) / 6.

      return
end Function theta

real(r4) Function kappaf(tf,sf,rf,prsf,pref,kkff)
implicit none
integer(i4),intent(in) :: kkff
real(r4),intent(in)    :: tf
real(r4),intent(in)    :: sf
real(r4),intent(in)    :: rf
real(r4),intent(in)    :: prsf
real(r4),intent(in)    :: pref

real(r4)               :: kappaf1
integer(i4)            :: kkf
real(r4)               :: r
real(r4)               :: s
real(r4)               :: t
real(r4)               :: prs
real(r4), parameter    :: qthref=1.0e3
real(r4), parameter    :: sclkap=1.e-11

!###coefficients for kappa^(theta)
!###new values (w.r.t. t-toff,s-soff,prs) from Shan Sun, Sep.2004
!###1=Arctic/Antarctic; 2=Atlantic; 3=Mediterranean

real(r4),parameter,dimension(3) ::                             &
 toff = (/  0.0,             3.0,            13.0 /)           &
,soff = (/ 34.5,            35.0,            38.5 /)           &
,qttt = (/ -3.03869354E-05, -3.03869352E-05, -3.03869353E-05 /)&
,qtt  = (/  4.56625601E-03,  4.29277358E-03,  3.38116552E-03 /)&
,qt   = (/ -2.88801209E-01, -2.61828868E-01, -1.81335007E-01 /)&
,qs   = (/ -1.08670290E-01, -1.05131061E-01, -9.33336309E-02 /)&
,qst  = (/  7.90503772E-04,  7.71096940E-04,  1.07270585E-03 /)&
,qpt  = (/  1.07813750E-09,  1.00638435E-09,  7.57239852E-10 /)&
,qpst = (/  1.41541548E-11,  1.48598578E-11,  3.89226107E-12 /)&
,qptt = (/ -1.31383708E-11, -1.31383707E-11, -1.31383708E-11 /)


!###thermobaric compressibility coefficient (integral from prs to pref)
!###Sun et.al. (1999) JPO 29 pp 2719-2729. kappaf1 used internally to 
!###simplify offsetting T and S,always invoke via kappaf. offset limits 
!###based on stability estimates from: Hallberg (2005) Ocean Modelling 8 pp 279-300.
!### t: potential temperature; s: psu; prs: pressure; kkf: ref.state
!### example: kappaf(4.5,34.5,1.e7,1) =  0.11411243
!### example: kappaf(4.5,34.5,1.e7,2) =  0.03091669
!### example: kappaf(4.5,34.5,1.e7,3) = -0.06423524

!kappaf1(t,s,prs,kkf)=(1.e-11/1.0e-3)*(prs-pref)*( s*( qs(kkf)+t* qst(kkf) ) + & 
!t*(qt(kkf)+t*(qtt(kkf)+t*qttt(kkf))+0.5*(prs+pref)*(qpt(kkf)+s*qpst(kkf)+t*qptt(kkf))) )
!kappaf = kappaf1(max(-1.5,tf-toff(kkff) ),max(-4.0,min(2.0,sf-soff(kkff))),prsf,kkff)

kappaf1(t,s,r,prs,kkf)=(r+qthref)*(exp(sclkap*(prs-pref)*( s*( qs(kkf)+t*qst(kkf) ) + &
t*( qt(kkf)+t*(qtt(kkf)+t*qttt(kkf))+0.5*(prs+pref)*(qpt(kkf)+s*qpst(kkf)+t*qptt(kkf)) ) ) )-1.0)

kappaf=kappaf1(max(-1.2,tf-toff(kkff) ),max(-3.0,min(1.5,sf-soff(kkff))),rf,prsf,kkff)
end function kappaf

real(r4) Function sig(t,s,sigver)

! --- calculate density using the equation of state.
implicit none
integer(i4) :: sigver
real(r4)    :: t,s
real(r4)    :: c1,c2,c3,c4,c5,c6,c7,c8,c9

!!Alex add sigvar 6
real(r4)    :: c001,c002,c003,c004,c005,c006,c007,c008,c009
real(r4)    :: c010,c011,c012,c013,c014,c015,c016,c017,c018,c019,c020
real(r4)    :: c021,c022,c023,c024,c025,c101,c103,c105,c108,c109,c111
real(r4)    :: prs2pdb,pref,rpdb,sig_n,sig_d,sig_q

if     (sigver.eq.1) then
! ---   !7-term sigma-0
        c1=-1.364710E-01   !const. coefficent
        c2= 4.681810E-02   !T      coefficent
        c3= 8.070040E-01   !S   coefficent
        c4=-7.453530E-03   !T^2    coefficent
        c5=-2.944180E-03   !T  S   coefficent
        c6= 3.435700E-05   !T^3    coefficent
        c7= 3.486580E-05   !T^2S   coefficent
        c8= 0.0            !S^2 coefficent
        c9= 0.0            !TS^2 coefficent
      elseif (sigver.eq.2) then
! ---   !7-term sigma-2
        c1= 9.770930E+00   !const. coefficent
        c2=-2.264930E-02   !T      coefficent
        c3= 7.898790E-01   !   S   coefficent
        c4=-6.432050E-03   !T^2    coefficent
        c5=-2.629830E-03   !T  S   coefficent
        c6= 2.758350E-05   !T^3    coefficent
        c7= 3.152350E-05   !T^2S   coefficent
        c8= 0.0            !   S^2 coefficent
        c9= 0.0            !T  S^2 coefficent
      elseif (sigver.eq.3) then
! ---   !9-term sigma-0
        c1=-4.311829E-02   !const. coefficent
        c2= 5.429948E-02   !T      coefficent
        c3= 8.011774E-01   !   S   coefficent
        c4=-7.641336E-03   !T^2    coefficent
        c5=-3.258442E-03   !T  S   coefficent
        c6= 3.757643E-05   !T^3    coefficent
        c7= 3.630361E-05   !T^2S   coefficent
        c8= 8.675546E-05   !   S^2 coefficent
        c9= 3.995086E-06   !T  S^2 coefficent
      elseif (sigver.eq.4) then
! ---   !9-term sigma-2
        c1= 9.903308E+00   !const. coefficent
        c2=-1.618075E-02   !T      coefficent
        c3= 7.819166E-01   !   S   coefficent
        c4=-6.593939E-03   !T^2    coefficent
        c5=-2.896464E-03   !T  S   coefficent
        c6= 3.038697E-05   !T^3    coefficent
        c7= 3.266933E-05   !T^2S   coefficent
        c8= 1.180109E-04   !   S^2 coefficent
        c9= 3.399511E-06   !T  S^2 coefficent
      elseif (sigver.eq.6) then
        c001= 9.9984085444849347d+02       !num. constant    coefficent
        c002= 7.3471625860981584d+00       !num.    T        coefficent
        c003=-5.3211231792841769d-02       !num.    T^2      coefficent
        c004= 3.6492439109814549d-04       !num.    T^3      coefficent
        c005= 2.5880571023991390d+00       !num.       S     coefficent
        c006= 6.7168282786692355d-03       !num.    T  S     coefficent
        c007= 1.9203202055760151d-03       !num.       S^2   coefficent
        c008= 1.0000000000000000d+00       !den. constant    coefficent
        c009= 7.2815210113327091d-03       !den.    T        coefficent
        c010=-4.4787265461983921d-05       !den.    T^2      coefficent
        c011= 3.3851002965802430d-07       !den.    T^3      coefficent
        c012= 1.3651202389758572d-10       !den.    T^4      coefficent
        c013= 1.7632126669040377d-03       !den.       S     coefficent
        c014= 8.8066583251206474d-06       !den.    T  S     coefficent
        c015= 1.8832689434804897d-10       !den.    T^3S     coefficent
        c016= 5.7463776745432097d-06       !den.    T  S^1.5 coefficent
        c017= 1.4716275472242334d-09       !den.    T^3S^1.5 coefficent
        c018= 1.1798263740430364d-02       !num. P           coefficent
        c019= 9.8920219266399117d-08       !num. P  T^2      coefficent
        c020= 4.6996642771754730d-06       !num. P     S     coefficent
        c021= 2.5862187075154352d-08       !num. P^2         coefficent
        c022= 3.2921414007960662d-12       !num. P^2T^2      coefficent
        c023= 6.7103246285651894d-06       !den. P           coefficent
        c024= 2.4461698007024582d-17       !den. P^2T^3      coefficent
        c025= 9.1534417604289062d-18       !den. P^3T        coefficent
        prs2pdb=1.d-4
        pref=2000.d4
        rpdb=pref*prs2pdb
! --- coefficients for 17-term rational function sig() at rpdb.
        c101=c001+(c018-c021*rpdb)*rpdb   !num. constant    coefficent
        c103=c003+(c019-c022*rpdb)*rpdb   !num.    T^2      coefficent
        c105=c005+c020*rpdb               !num.       S     coefficent
        c108=c008+c023*rpdb               !den. constant    coefficent
        c109=c009-c025*rpdb**3            !den.    T        coefficent
        c111=c011-c024*rpdb**2            !den.    T^3      coefficent
      else
! ---   !unknown
        c1= 0.0            !const. coefficent
        c2= 0.0            !T      coefficent
        c3= 0.0            !   S   coefficent
        c4= 0.0            !T^2    coefficent
        c5= 0.0            !T  S   coefficent
        c6= 0.0            !T^3    coefficent
        c7= 0.0            !T^2S   coefficent
        c8= 0.0            !   S^2 coefficent
        c9= 0.0            !T  S^2 coefficent
      endif
      if (sigver.le.4) then
         sig=(c1+s*(c3+s* c8)+t*(c2+s*(c5+s*c9)+t*(c4+s*c7+t*c6)))
      elseif (sigver.eq.6) then
        sig_n = c101 + t*(c002+t*(c103+t*c004)) + &
                            s*(c105-t*c006+s*c007)
        sig_d = c108 + t*(c109+t*(c010+t*(c111+t*c012))) + &
                            s*(c013-t*(c014+t*t*c015) + &
          sqrt(max(0.0,s))*(c016+t*t*c017))
        sig_q = 1.0 /sig_d
        sig   = sig_n*sig_q - 1000.0
      endif
return
end function sig


real(r4) Function dsig(t,s,dt,ds,sigver)

! --- calculate density using the equation of state.
implicit none
integer(i4) :: sigver
real(r4)    :: t,s,dt,ds
real(r4)    :: c1,c2,c3,c4,c5,c6,c7,c8,c9

if     (sigver.eq.1) then
! ---   !7-term sigma-0
        c1=-1.364710E-01   !const. coefficent
        c2= 4.681810E-02   !T      coefficent
        c3= 8.070040E-01   !S   coefficent
        c4=-7.453530E-03   !T^2    coefficent
        c5=-2.944180E-03   !T  S   coefficent
        c6= 3.435700E-05   !T^3    coefficent
        c7= 3.486580E-05   !T^2S   coefficent
        c8= 0.0            !S^2 coefficent
        c9= 0.0            !TS^2 coefficent
      elseif (sigver.eq.2) then
! ---   !7-term sigma-2
        c1= 9.770930E+00   !const. coefficent
        c2=-2.264930E-02   !T      coefficent
        c3= 7.898790E-01   !   S   coefficent
        c4=-6.432050E-03   !T^2    coefficent
        c5=-2.629830E-03   !T  S   coefficent
        c6= 2.758350E-05   !T^3    coefficent
        c7= 3.152350E-05   !T^2S   coefficent
        c8= 0.0            !   S^2 coefficent
        c9= 0.0            !T  S^2 coefficent
      elseif (sigver.eq.3) then
! ---   !9-term sigma-0
        c1=-4.311829E-02   !const. coefficent
        c2= 5.429948E-02   !T      coefficent
        c3= 8.011774E-01   !   S   coefficent
        c4=-7.641336E-03   !T^2    coefficent
        c5=-3.258442E-03   !T  S   coefficent
        c6= 3.757643E-05   !T^3    coefficent
        c7= 3.630361E-05   !T^2S   coefficent
        c8= 8.675546E-05   !   S^2 coefficent
        c9= 3.995086E-06   !T  S^2 coefficent
      elseif (sigver.eq.4) then
! ---   !9-term sigma-2
        c1= 9.903308E+00   !const. coefficent
        c2=-1.618075E-02   !T      coefficent
        c3= 7.819166E-01   !   S   coefficent
        c4=-6.593939E-03   !T^2    coefficent
        c5=-2.896464E-03   !T  S   coefficent
        c6= 3.038697E-05   !T^3    coefficent
        c7= 3.266933E-05   !T^2S   coefficent
        c8= 1.180109E-04   !   S^2 coefficent
        c9= 3.399511E-06   !T  S^2 coefficent
      else
! ---   !unknown
        c1= 0.0            !const. coefficent
        c2= 0.0            !T      coefficent
        c3= 0.0            !   S   coefficent
        c4= 0.0            !T^2    coefficent
        c5= 0.0            !T  S   coefficent
        c6= 0.0            !T^3    coefficent
        c7= 0.0            !T^2S   coefficent
        c8= 0.0            !   S^2 coefficent
        c9= 0.0            !T  S^2 coefficent
      endif
      dsig=(c2+c5*s+2.0*c4*s+2.0*c7*s+c9*s*s+3*c6*t*t)*dt + (c3+c5*t+2.0*c8*s+c7*t*t+2.0*c9*t)*ds 
                     
return
end function dsig


real(r4) Function tofsig(r,s,sigver)
implicit none
integer(i4) :: sigver
real(r4)    :: r,s
real(r4)    :: c1,c2,c3,c4,c5,c6,c7,c8,c9

if     (sigver.eq.1) then
! ---   !7-term sigma-0
        c1=-1.364710E-01   !const. coefficent
        c2= 4.681810E-02   !T      coefficent
        c3= 8.070040E-01   !S   coefficent
        c4=-7.453530E-03   !T^2    coefficent
        c5=-2.944180E-03   !T  S   coefficent
        c6= 3.435700E-05   !T^3    coefficent
        c7= 3.486580E-05   !T^2S   coefficent
        c8= 0.0            !S^2 coefficent
        c9= 0.0            !TS^2 coefficent
      elseif (sigver.eq.2) then
! ---   !7-term sigma-2
        c1= 9.770930E+00   !const. coefficent
        c2=-2.264930E-02   !T      coefficent
        c3= 7.898790E-01   !   S   coefficent
        c4=-6.432050E-03   !T^2    coefficent
        c5=-2.629830E-03   !T  S   coefficent
        c6= 2.758350E-05   !T^3    coefficent
        c7= 3.152350E-05   !T^2S   coefficent
        c8= 0.0            !   S^2 coefficent
        c9= 0.0            !T  S^2 coefficent
      elseif (sigver.eq.3) then
! ---   !9-term sigma-0
        c1=-4.311829E-02   !const. coefficent
        c2= 5.429948E-02   !T      coefficent
        c3= 8.011774E-01   !   S   coefficent
        c4=-7.641336E-03   !T^2    coefficent
        c5=-3.258442E-03   !T  S   coefficent
        c6= 3.757643E-05   !T^3    coefficent
        c7= 3.630361E-05   !T^2S   coefficent
        c8= 8.675546E-05   !   S^2 coefficent
        c9= 3.995086E-06   !T  S^2 coefficent
      elseif (sigver.eq.4) then
! ---   !9-term sigma-2
        c1= 9.903308E+00   !const. coefficent
        c2=-1.618075E-02   !T      coefficent
        c3= 7.819166E-01   !   S   coefficent
        c4=-6.593939E-03   !T^2    coefficent
        c5=-2.896464E-03   !T  S   coefficent
        c6= 3.038697E-05   !T^3    coefficent
        c7= 3.266933E-05   !T^2S   coefficent
        c8= 1.180109E-04   !   S^2 coefficent
        c9= 3.399511E-06   !T  S^2 coefficent
      else
! ---   !unknown
        c1= 0.0            !const. coefficent
        c2= 0.0            !T      coefficent
        c3= 0.0            !   S   coefficent
        c4= 0.0            !T^2    coefficent
        c5= 0.0            !T  S   coefficent
        c6= 0.0            !T^3    coefficent
        c7= 0.0            !T^2S   coefficent
        c8= 0.0            !   S^2 coefficent
        c9= 0.0            !T  S^2 coefficent
      endif

! --- temp (deg c) as a function of sigma and salinity (psu)
! --- find a cubic polynominal root of t**3+a2*t**2+a1*t+a0=0
      tofsig=-cubrl(r,s,c1,c2,c3,c4,c5,c6,c7,c8,c9)+sqrt(3.0)*cubim(r,s,c1,c2,c3,c4,c5,c6,c7,c8,c9)-a3rd*a2(s,c1,c2,c3,c4,c5,c6,c7,c8,c9)

return
end function tofsig

real(r4) Function sofsig(r,t,sigver)
implicit none
integer(i4) :: sigver
real(r4)    :: t,r
real(r4)    :: c1,c2,c3,c4,c5,c6,c7,c8,c9

if     (sigver.eq.1) then
! ---   !7-term sigma-0
        c1=-1.364710E-01   !const. coefficent
        c2= 4.681810E-02   !T      coefficent
        c3= 8.070040E-01   !S   coefficent
        c4=-7.453530E-03   !T^2    coefficent
        c5=-2.944180E-03   !T  S   coefficent
        c6= 3.435700E-05   !T^3    coefficent
        c7= 3.486580E-05   !T^2S   coefficent
        c8= 0.0            !S^2 coefficent
        c9= 0.0            !TS^2 coefficent
      elseif (sigver.eq.2) then
! ---   !7-term sigma-2
        c1= 9.770930E+00   !const. coefficent
        c2=-2.264930E-02   !T      coefficent
        c3= 7.898790E-01   !   S   coefficent
        c4=-6.432050E-03   !T^2    coefficent
        c5=-2.629830E-03   !T  S   coefficent
        c6= 2.758350E-05   !T^3    coefficent
        c7= 3.152350E-05   !T^2S   coefficent
        c8= 0.0            !   S^2 coefficent
        c9= 0.0            !T  S^2 coefficent
      elseif (sigver.eq.3) then
! ---   !9-term sigma-0
        c1=-4.311829E-02   !const. coefficent
        c2= 5.429948E-02   !T      coefficent
        c3= 8.011774E-01   !   S   coefficent
        c4=-7.641336E-03   !T^2    coefficent
        c5=-3.258442E-03   !T  S   coefficent
        c6= 3.757643E-05   !T^3    coefficent
        c7= 3.630361E-05   !T^2S   coefficent
        c8= 8.675546E-05   !   S^2 coefficent
        c9= 3.995086E-06   !T  S^2 coefficent
      elseif (sigver.eq.4) then
      ! ---   !9-term sigma-2
        c1= 9.903308E+00   !const. coefficent
        c2=-1.618075E-02   !T      coefficent
        c3= 7.819166E-01   !   S   coefficent
        c4=-6.593939E-03   !T^2    coefficent
        c5=-2.896464E-03   !T  S   coefficent
        c6= 3.038697E-05   !T^3    coefficent
        c7= 3.266933E-05   !T^2S   coefficent
        c8= 1.180109E-04   !   S^2 coefficent
        c9= 3.399511E-06   !T  S^2 coefficent
      else
! ---   !unknown
        c1= 0.0            !const. coefficent
        c2= 0.0            !T      coefficent
        c3= 0.0            !   S   coefficent
        c4= 0.0            !T^2    coefficent
        c5= 0.0            !T  S   coefficent
        c6= 0.0            !T^3    coefficent
        c7= 0.0            !T^2S   coefficent
        c8= 0.0            !   S^2 coefficent
        c9= 0.0            !T  S^2 coefficent
      endif

sofsig=(2.0*sofsig_c(r,t,c1,c2,c3,c4,c5,c6,c7,c8,c9))&
/(-sofsig_b(r,t,c1,c2,c3,c4,c5,c6,c7,c8,c9) &
-sign(sqrt(sofsig_b(r,t,c1,c2,c3,c4,c5,c6,c7,c8,c9)**2 &
-4.0*sofsig_a(r,t,c1,c2,c3,c4,c5,c6,c7,c8,c9)*sofsig_c(r,t,c1,c2,c3,c4,c5,c6,c7,c8,c9))&
,sofsig_b(r,t,c1,c2,c3,c4,c5,c6,c7,c8,c9)))

return
end function sofsig


real(r4) function cubq(s,c1,c2,c3,c4,c5,c6,c7,c8,c9)
real(r4) :: s
real(r4) :: c1,c2,c3,c4,c5,c6,c7,c8,c9
real(r4) :: rc6
cubq=a3rd*a1(s,c1,c2,c3,c4,c5,c6,c7,c8,c9)-(a3rd*a2(s,c1,c2,c3,c4,c5,c6,c7,c8,c9))**2
end function cubq

real(r4) function cubr(r,s,c1,c2,c3,c4,c5,c6,c7,c8,c9)
real(r4) :: r,s
real(r4) :: c1,c2,c3,c4,c5,c6,c7,c8,c9
real(r4) :: rc6
cubr=a3rd*(0.5*a1(s,c1,c2,c3,c4,c5,c6,c7,c8,c9)*a2(s,c1,c2,c3,c4,c5,c6,c7,c8,c9)&
-1.5*a0(r,s,c1,c2,c3,c4,c5,c6,c7,c8,c9))-(a3rd*a2(s,c1,c2,c3,c4,c5,c6,c7,c8,c9))**3
end function cubr

real(r4) function cuban(r,s,c1,c2,c3,c4,c5,c6,c7,c8,c9)
real(r4) :: r,s
real(r4) :: c1,c2,c3,c4,c5,c6,c7,c8,c9
real(r4) :: rc6
cuban=a3rd*atan2(sqrt(max(0.0,-(cubq(s,c1,c2,c3,c4,c5,c6,c7,c8,c9)**3+cubr(r,s,c1,c2,c3,c4,c5,c6,c7,c8,c9)**2))),cubr(r,s,c1,c2,c3,c4,c5,c6,c7,c8,c9))
end function cuban

real(r4) function cubrl(r,s,c1,c2,c3,c4,c5,c6,c7,c8,c9)
real(r4) :: r,s
real(r4) :: c1,c2,c3,c4,c5,c6,c7,c8,c9
real(r4) :: rc6
cubrl=sqrt(-cubq(s,c1,c2,c3,c4,c5,c6,c7,c8,c9))*cos(cuban(r,s,c1,c2,c3,c4,c5,c6,c7,c8,c9))
end function cubrl

real(r4) function cubim(r,s,c1,c2,c3,c4,c5,c6,c7,c8,c9)
real(r4) :: r,s
real(r4) :: c1,c2,c3,c4,c5,c6,c7,c8,c9
real(r4) :: rc6
cubim=sqrt(-cubq(s,c1,c2,c3,c4,c5,c6,c7,c8,c9))*sin(cuban(r,s,c1,c2,c3,c4,c5,c6,c7,c8,c9))
end function cubim

real(r4) function a0(r,s,c1,c2,c3,c4,c5,c6,c7,c8,c9)
real(r4) :: r,s
real(r4) :: c1,c2,c3,c4,c5,c6,c7,c8,c9
real(r4) :: rc6

rc6=1.0/c6
a0=(c1+s*(c3+s*c8)-r)*rc6  !constant  coefficient
end function a0

real(r4) function a1(s,c1,c2,c3,c4,c5,c6,c7,c8,c9)
real(r4) :: s
real(r4) :: c1,c2,c3,c4,c5,c6,c7,c8,c9
real(r4) :: rc6

rc6=1.0/c6
a1=(c2+s*(c5+s*c9)  )*rc6  !linear    coefficient
end function a1

real(r4) function a2(s,c1,c2,c3,c4,c5,c6,c7,c8,c9)
real(r4) :: s
real(r4) :: c1,c2,c3,c4,c5,c6,c7,c8,c9
real(r4) :: rc6

rc6=1.0/c6
a2  =(c4+s*c7)*rc6  !quadratic coefficient
end function a2

real(r4) function sofsig_a(r,t,c1,c2,c3,c4,c5,c6,c7,c8,c9)
real(r4) :: t,r
real(r4) :: c1,c2,c3,c4,c5,c6,c7,c8,c9
real(r4) :: rc6

sofsig_a=(c8+t* c9)   !quadratic coefficient
end function sofsig_a

real(r4) function sofsig_b(r,t,c1,c2,c3,c4,c5,c6,c7,c8,c9)
real(r4) :: r,t
real(r4) :: c1,c2,c3,c4,c5,c6,c7,c8,c9
real(r4) :: rc6

sofsig_b=(c3+t*(c5+t* c7))         !linear    coefficient
end function sofsig_b

real(r4) function sofsig_c(r,t,c1,c2,c3,c4,c5,c6,c7,c8,c9)
real(r4) :: r,t
real(r4) :: c1,c2,c3,c4,c5,c6,c7,c8,c9
real(r4) :: rc6
sofsig_c=(c1+t*(c2+t*(c4+t*c6))-r) !constant  coefficient
end function sofsig_c

end module mseawater 
